package com.siapics.server

import org.jetbrains.exposed.sql.*
import org.jetbrains.exposed.sql.transactions.transaction
import java.io.File
import kotlin.streams.asSequence

class Database {

    object Images : Table() {
        val id: Column<Int> = integer("id").autoIncrement().primaryKey()
        val externalId: Column<String> = text("externalId")
        val filename: Column<String?> = text("filename").nullable()
    }

    // TODO: use a different path (something thats not in the git repo)
    private val DatabaseFile = File("database").absolutePath
    private val CreateImageIdLock = Object()

    private val db = org.jetbrains.exposed.sql.Database.connect("jdbc:h2:file:$DatabaseFile", driver = "org.h2.Driver")

    init {
        transaction(db) {
            SchemaUtils.createMissingTablesAndColumns(Images)
        }
    }

    fun generateRandomImageId(): String {
        synchronized(CreateImageIdLock) {
            var randomId = ""
            while (randomId.isEmpty()) {
                val characters = "abcdefghijklmnopqrstuvwxyz0123456789"
                randomId = java.util.Random().ints(6, 0, characters.length)
                        .asSequence()
                        .map(characters::get)
                        .joinToString("")
                transaction(db) {
                    SchemaUtils.create(Images)
                    if (Images.select { Images.externalId.eq(randomId) }.count() > 0) {
                        randomId = ""
                    }
                }
            }

            transaction(db) {
                Images.insert {
                    it[externalId] = randomId
                    it[filename] = null
                }
            }
            return randomId
        }
    }

    fun updateImage(imageId: String, filename: String?) {
        transaction {
            Images.update({ Images.externalId.eq(imageId)}) {
                it[Images.filename] = filename
            }
        }
    }

    fun getImage(imageId: String): ResultRow {
        return transaction(db) {
            Images.select { Images.externalId.eq(imageId)}.first()
        }
    }
}