package com.siapics.server

import io.minio.MinioClient
import spark.ModelAndView
import spark.Request
import spark.Response
import spark.Spark
import spark.template.velocity.VelocityTemplateEngine
import java.io.File
import java.nio.file.Files
import javax.servlet.MultipartConfigElement

object Main {

    private val AllowedExtensions = setOf("png", "jpeg", "jpg", "svg", "gif", "bmp")

    // TODO: read keys from env
    private val minioClient = MinioClient("http://195.201.141.43:9000",
            "ZVLLMKIJD1ULKOF3KGRG", "pLPB+LfzSFeUt2bKJ4WuYkCLtU8yvSLFe6bTgqp9")
    private val database = Database()

    @JvmStatic fun main(args: Array<String>) {
        // Create bucket if it doesn't exist yet.
        if (!minioClient.bucketExists("siapics")) {
            minioClient.makeBucket("siapics")
        }

        Spark.get("/", { request: Request, response: Response ->
            ModelAndView(mutableMapOf("foo" to "test"), "index.vm")
        }, VelocityTemplateEngine())

        Spark.post("/", this::uploadFile)

        Spark.get("/view/:id", { request: Request, response: Response ->
            val image = database.getImage(request.params("id"))
            ModelAndView(mutableMapOf("filename" to image[Database.Images.filename]), "view.vm")
        }, VelocityTemplateEngine())

        Spark.get("/image/:id", { request: Request, response: Response ->
            val inputStream = minioClient.getObject("siapics", request.params("id"))
            inputStream.copyTo(response.raw().outputStream)
        })
    }

    private fun uploadFile(request: Request, response: Response) {
        if (request.contentLength() > 5 * 1024 * 1024) {
            Spark.halt("File is too big, maximum 5 MB")
        }

        val imageId = database.generateRandomImageId()
        val tempFile = copyUploadToTempFile(imageId, request)

        val filename = getFileName(request)
        val extension = filename?.split(".")?.last()
        if (!AllowedExtensions.contains(extension)) {
            tempFile.delete()
            Spark.halt("Invalid file type. Supported types are: ${AllowedExtensions.joinToString()}")
        }
        database.updateImage(imageId, filename)

        minioClient.putObject("siapics", imageId, tempFile.absolutePath, request.contentType())
        tempFile.delete()
        response.redirect("/view/$imageId", 303)
    }

    private fun copyUploadToTempFile(imageId: String, request: Request): File {
        val tempFile = Files.createTempFile(null, imageId).toFile()
        request.attribute("org.eclipse.jetty.multipartConfig", MultipartConfigElement("/tmp"))
        request.raw().getPart("uploaded_file").inputStream.use { inputStream ->
            tempFile.outputStream().use { outputStream ->
                inputStream.copyTo(outputStream)
            }
        }
        return tempFile
    }

    /**
     * Reads the original filename of the uploaded file.
     *
     * From https://stackoverflow.com/a/30290614
     */
    private fun getFileName(request: Request): String? {
        val part = request.raw().getPart("uploaded_file")
        for (cd in part.getHeader("content-disposition").split(";")) {
            if (cd.trim({ it <= ' ' }).startsWith("filename")) {
                return cd.substring(cd.indexOf('=') + 1).trim({ it <= ' ' }).replace("\"", "")
            }
        }
        return null
    }
}
